@@ -89,25 +89,40 @@ class Dataset1D(object):
 
 class Dataset2D(object):
     """2D dataset for ImportPlugin or DatasetPlugin."""
-    def __init__(self, name, data=[[]], rangex=None, rangey=None):
+    def __init__(self, name, data=[[]], rangex=None, rangey=None,
+                 xedge=None, yedge=None,
+                 xcent=None, ycent=None):
         """2D dataset.
         name: name of dataset
         data: 2D numpy array of values or list of lists of floats
         rangex: optional tuple with X range of data (min, max)
         rangey: optional tuple with Y range of data (min, max)
+        xedge: x values for grid (instead of rangex)
+        yedge: y values for grid (instead of rangey)
+        xcent: x values for pixel centres (instead of rangex)
+        ycent: y values for pixel centres (instead of rangey)
         """
         self.name = name
-        self.update(data=data, rangex=rangex, rangey=rangey)
+        self.update(data=data, rangex=rangex, rangey=rangey,
+                    xedge=xedge, yedge=yedge,
+                    xcent=xcent, ycent=ycent)
 
-    def update(self, data=[[]], rangex=None, rangey=None):
+    def update(self, data=[[]], rangex=None, rangey=None,
+               xedge=None, yedge=None,
+               xcent=None, ycent=None):
         self.data = N.array(data, dtype=N.float64)
         self.rangex = rangex
         self.rangey = rangey
+        self.xedge = xedge
+        self.yedge = yedge
+        self.xcent = xcent
+        self.ycent = ycent
 
     def _null(self):
         """Empty data contents."""
         self.data = N.array([[]])
         self.rangex = self.rangey = (0, 1)
+        self.xedge = self.yedge = self.xcent = self.ycent = None
 
     def _makeVeuszDataset(self, manager):
         """Make a Veusz dataset from the plugin dataset."""
@@ -259,14 +274,15 @@ class DatasetPluginHelper(object):
         try:
             ds = self._doc.data[name]
         except KeyError:
-            raise DatasetPluginException("Unknown dataset '%s'" % name)
+            raise DatasetPluginException(_("Unknown dataset '%s'") % name)
 
         if ds.dimensions != dimensions:
             raise DatasetPluginException(
-                "Dataset '%s' does not have %i dimensions" % (name, dimensions))
+                _("Dataset '%s' does not have %i dimensions") % (
+                    name, dimensions))
         if ds.datatype != 'numeric':
             raise DatasetPluginException(
-                "Dataset '%s' is not a numerical dataset" % name)
+                _("Dataset '%s' is not a numerical dataset") % name)
 
         if isinstance(ds, document.DatasetDateTime):
             return DatasetDateTime(name, data=ds.data)
@@ -293,10 +309,10 @@ class DatasetPluginHelper(object):
         try:
             ds = self._doc.data[name]
         except KeyError:
-            raise DatasetPluginException("Unknown dataset '%s'" % name)
+            raise DatasetPluginException(_("Unknown dataset '%s'") % name)
         if ds.datatype == 'text':
             return DatasetText(name, ds.data)
-        raise DatasetPluginException("Dataset '%s' is not a text datset" % name)
+        raise DatasetPluginException(_("Dataset '%s' is not a text datset") % name)
 
 # internal object to synchronise datasets created by a plugin
 class DatasetPluginManager(object):
@@ -434,7 +450,7 @@ class _OneOutputDatasetPlugin(DatasetPlugin):
     def getDatasets(self, fields):
         """Returns single output dataset (self.dsout)."""
         if fields['ds_out'] == '':
-            raise DatasetPluginException('Invalid output dataset name')
+            raise DatasetPluginException(_('Invalid output dataset name'))
         self.dsout = Dataset1D(fields['ds_out'])
         return [self.dsout]
 
@@ -1538,9 +1554,9 @@ class ReBinXYPlugin(DatasetPlugin):
     def getDatasets(self, fields):
         """Return output datasets"""
         if fields['ds_yout'] == '':
-            raise DatasetPluginException('Invalid output Y dataset name')
+            raise DatasetPluginException(_('Invalid output Y dataset name'))
         if fields['ds_x'] != '' and fields['ds_xout'] == '':
-            raise DatasetPluginException('Invalid output X dataset name')
+            raise DatasetPluginException(_('Invalid output X dataset name'))
 
         self.dssout = out = [ Dataset1D(fields['ds_yout']) ]
         if fields['ds_xout'] != '':
@@ -1668,7 +1684,7 @@ class SortTextPlugin(_OneOutputDatasetPlugin):
     def getDatasets(self, fields):
         """Returns single output dataset (self.dsout)."""
         if fields['ds_out'] == '':
-            raise DatasetPluginException('Invalid output dataset name')
+            raise DatasetPluginException(_('Invalid output dataset name'))
         self.dsout = DatasetText(fields['ds_out'])
         return [self.dsout]
 
@@ -1790,7 +1806,7 @@ class Histogram2D(DatasetPlugin):
     def getDatasets(self, fields):
         """Returns single output dataset (self.dsout)."""
         if fields['ds_out'] == '':
-            raise DatasetPluginException('Invalid output dataset name')
+            raise DatasetPluginException(_('Invalid output dataset name'))
         self.dsout = Dataset2D(fields['ds_out'])
         return [self.dsout]
 
@@ -1819,7 +1835,7 @@ class ConvertNumbersToText(DatasetPlugin):
 
     def getDatasets(self, fields):
         if fields['ds_out'] == '':
-            raise DatasetPluginException('Invalid output dataset name')
+            raise DatasetPluginException(_('Invalid output dataset name'))
         self.dsout = DatasetText(fields['ds_out'])
         return [self.dsout]
 
