@@ -18,14 +18,14 @@
 
 """A database for default values of settings."""
 
+from __future__ import division, print_function
 import sys
 
-import veusz.qtall as qt4
+from .. import qtall as qt4
 
 def _(text, disambiguation=None, context="Preferences"):
     """Translate text."""
-    return unicode(
-        qt4.QCoreApplication.translate(context, text, disambiguation))
+    return qt4.QCoreApplication.translate(context, text, disambiguation)
 
 # default values to some settings in case the user does not have these
 defaultValues = {
@@ -134,19 +134,19 @@ class _SettingDB(object):
         s = qt4.QSettings(self.domain, self.product)
 
         for key in s.childKeys():
-            val = s.value(key).toString()
-            realkey = unicode(key).replace(self.sepchars, '/')
+            val = s.value(key)
+            realkey = key.replace(self.sepchars, '/')
 
             try:
-                self.database[realkey] = eval( unicode(val) )
+                self.database[realkey] = eval(val)
             except:
-                print >>sys.stderr, ('Error interpreting item "%s" in '
-                                     'settings file' % realkey)
+                print('Error interpreting item "%s" in '
+                      'settings file' % realkey, file=sys.stderr)
 
         # set any defaults which haven't been set
-        for key, value in defaultValues.iteritems():
+        for key in defaultValues:
             if key not in self.database:
-                self.database[key] = value
+                self.database[key] = defaultValues[key]
 
     def writeSettings(self):
         """Write the settings using QSettings.
@@ -158,19 +158,15 @@ class _SettingDB(object):
 
         # write each entry, keeping track of which ones haven't been written
         cleankeys = []
-        for key, value in self.database.iteritems():
+        for key in self.database:
             cleankey = key.replace('/', self.sepchars)
             cleankeys.append(cleankey)
 
-            # repr doesn't work on QStrings
-            if isinstance(value, qt4.QString):
-                value = unicode(value)
-
-            s.setValue(cleankey, qt4.QVariant(repr(value)))
+            s.setValue(cleankey, repr(self.database[key]))
 
         # now remove all the values which have been removed
         for key in list(s.childKeys()):
-            if unicode(key) not in cleankeys:
+            if key not in cleankeys:
                 s.remove(key)
 
     def get(self, key, defaultval=None):
@@ -212,4 +208,28 @@ def updateUILocale():
 
     qt4.QLocale.setDefault(uilocale)
 
+def ui_floattostring(f):
+    """Convert float to string with more precision."""
+    if 1e-4 <= abs(f) <= 1e5 or f == 0:
+        s = '%.14g' % f
+        # strip excess zeros to right
+        if s.find('.') >= 0:
+            s = s.rstrip('0').rstrip('.')
+    else:
+        s = '%.14e' % f
+        # split into mantissa/exponent and strip extra zeros, etc
+        mant, expon = s.split('e')
+        mant = mant.rstrip('0').rstrip('.')
+        expon = int(expon)
+        s = '%se%i' % (mant, expon)
+    # make decimal point correct for local
+    s = s.replace('.', uilocale.decimalPoint())
+    return s
+
+def ui_stringtofloat(s):
+    """Convert string to float, allowing for decimal point in different
+    locale."""
+    s = s.replace(uilocale.decimalPoint(), '.')
+    return float(s)
+
 updateUILocale()
