@@ -20,7 +20,6 @@
 positions."""
 
 from __future__ import division
-import re
 import numpy as N
 
 from ..compat import czip
@@ -1078,16 +1077,19 @@ class Axis(widget.Widget):
 
         texttorender is a list of (Renderer, QPen) tuples.
         """
-        drawntext = qt4.QPainterPath()
-        for r, pen in texttorender:
-            bounds = r.getBounds()
-            rect = qt4.QRectF(bounds[0], bounds[1], bounds[2]-bounds[0],
-                              bounds[3]-bounds[1])
+        overlaps = utils.RectangleOverlapTester()
 
-            if not drawntext.intersects(rect):
+        for r, pen in texttorender:
+            rect = r.getTightBounds()
+
+            if not overlaps.willOverlap(rect):
                 painter.setPen(pen)
                 r.render()
-                drawntext.addRect(rect)
+                overlaps.addRect(rect)
+
+            # debug
+            # poly = rect.makePolygon()
+            # painter.drawPolygon(poly)
 
     def _axisDraw(self, posn, parentposn, outerbounds, painter, phelper):
         """Internal drawing routine."""
@@ -1160,13 +1162,18 @@ class Axis(widget.Widget):
                 c1, c2 = c2, c1
                 c1delt, c2delt = c2delt, c1delt
 
+            round1 = utils.round2delt(c1, c1delt)
+            round2 = utils.round2delt(c2, c2delt)
+
             ops = []
-            if s.min == 'Auto' or not N.allclose(c1, s.min, rtol=1e-8):
+            if ( (s.min == 'Auto' or not N.allclose(c1, s.min, rtol=1e-8))
+                 and N.isfinite(round1) ):
                 ops.append( document.OperationSettingSet(
-                        s.get('min'), utils.round2delt(c1, c1delt)) )
-            if s.max == 'Auto' or not N.allclose(c2, s.max, rtol=1e-8):
+                        s.get('min'), round1) )
+            if ( (s.max == 'Auto' or not N.allclose(c2, s.max, rtol=1e-8))
+                 and N.isfinite(round2) ):
                 ops.append( document.OperationSettingSet(
-                        s.get('max'), utils.round2delt(c2, c2delt)) )
+                        s.get('max'), round2) )
 
             self.document.applyOperation(
                 document.OperationMultiple(ops, descr=_('zoom axis')))
